@@ -49,25 +49,22 @@ class IntroduceBufferAuxiliaryVars(CythonTransform):
         # For all buffers, insert extra variables in the scope.
         # The variables are also accessible from the buffer_info
         # on the buffer entry
-        bufvars = [entry for name, entry
-                   in scope.entries.iteritems()
-                   if entry.type.is_buffer]
+        scope_items = scope.entries.items()
+        bufvars = [entry for name, entry in scope_items if entry.type.is_buffer]
         if len(bufvars) > 0:
             bufvars.sort(key=lambda entry: entry.name)
             self.buffers_exists = True
 
-        memviewslicevars = [entry for name, entry
-                in scope.entries.iteritems()
-                if entry.type.is_memoryviewslice]
+        memviewslicevars = [entry for name, entry in scope_items if entry.type.is_memoryviewslice]
         if len(memviewslicevars) > 0:
             self.buffers_exists = True
 
 
-        for (name, entry) in scope.entries.iteritems():
+        for (name, entry) in scope_items:
             if name == 'memoryview' and isinstance(entry.utility_code_definition, CythonUtilityCode):
                 self.using_memoryview = True
                 break
-
+        del scope_items
 
         if isinstance(node, ModuleNode) and len(bufvars) > 0:
             # for now...note that pos is wrong
@@ -143,13 +140,14 @@ def analyse_buffer_options(globalpos, env, posargs, dictargs, defaults=None, nee
     if defaults is None:
         defaults = buffer_defaults
 
-    posargs, dictargs = Interpreter.interpret_compiletime_options(posargs, dictargs, type_env=env, type_args = (0,'dtype'))
+    posargs, dictargs = Interpreter.interpret_compiletime_options(
+        posargs, dictargs, type_env=env, type_args=(0, 'dtype'))
 
     if len(posargs) > buffer_positional_options_count:
         raise CompileError(posargs[-1][1], ERR_BUF_TOO_MANY)
 
     options = {}
-    for name, (value, pos) in dictargs.iteritems():
+    for name, (value, pos) in dictargs.items():
         if not name in buffer_options:
             raise CompileError(pos, ERR_BUF_OPTION_UNKNOWN % name)
         options[name] = value
@@ -256,7 +254,7 @@ class BufferEntry(object):
             defcode = code.globalstate['utility_code_def']
             funcgen(protocode, defcode, name=funcname, nd=nd)
 
-        buf_ptr_type_code = self.buf_ptr_type.declaration_code("")
+        buf_ptr_type_code = self.buf_ptr_type.empty_declaration_code()
         ptrcode = "%s(%s, %s, %s)" % (funcname, buf_ptr_type_code, self.buf_ptr,
                                       ", ".join(params))
         return ptrcode
@@ -415,6 +413,7 @@ def put_assign_to_buffer(lhs_cname, rhs_cname, buf_entry,
 
     code.putln("}") # Release stack
 
+
 def put_buffer_lookup_code(entry, index_signeds, index_cnames, directives,
                            pos, code, negative_indices, in_nogil_context):
     """
@@ -435,21 +434,21 @@ def put_buffer_lookup_code(entry, index_signeds, index_cnames, directives,
     if directives['boundscheck']:
         # Check bounds and fix negative indices.
         # We allocate a temporary which is initialized to -1, meaning OK (!).
-        # If an error occurs, the temp is set to the dimension index the
-        # error is occuring at.
-        tmp_cname = code.funcstate.allocate_temp(PyrexTypes.c_int_type, manage_ref=False)
-        code.putln("%s = -1;" % tmp_cname)
-        for dim, (signed, cname, shape) in enumerate(zip(index_signeds, index_cnames,
-                                                         entry.get_buf_shapevars())):
+        # If an error occurs, the temp is set to the index dimension the
+        # error is occurring at.
+        failed_dim_temp = code.funcstate.allocate_temp(PyrexTypes.c_int_type, manage_ref=False)
+        code.putln("%s = -1;" % failed_dim_temp)
+        for dim, (signed, cname, shape) in enumerate(zip(index_signeds, index_cnames, entry.get_buf_shapevars())):
             if signed != 0:
                 # not unsigned, deal with negative index
                 code.putln("if (%s < 0) {" % cname)
                 if negative_indices:
                     code.putln("%s += %s;" % (cname, shape))
                     code.putln("if (%s) %s = %d;" % (
-                        code.unlikely("%s < 0" % cname), tmp_cname, dim))
+                        code.unlikely("%s < 0" % cname),
+                        failed_dim_temp, dim))
                 else:
-                    code.putln("%s = %d;" % (tmp_cname, dim))
+                    code.putln("%s = %d;" % (failed_dim_temp, dim))
                 code.put("} else ")
             # check bounds in positive direction
             if signed != 0:
@@ -458,7 +457,7 @@ def put_buffer_lookup_code(entry, index_signeds, index_cnames, directives,
                 cast = "(size_t)"
             code.putln("if (%s) %s = %d;" % (
                 code.unlikely("%s >= %s%s" % (cname, cast, shape)),
-                              tmp_cname, dim))
+                failed_dim_temp, dim))
 
         if in_nogil_context:
             code.globalstate.use_utility_code(raise_indexerror_nogil)
@@ -467,15 +466,14 @@ def put_buffer_lookup_code(entry, index_signeds, index_cnames, directives,
             code.globalstate.use_utility_code(raise_indexerror_code)
             func = '__Pyx_RaiseBufferIndexError'
 
-        code.putln("if (%s) {" % code.unlikely("%s != -1" % tmp_cname))
-        code.putln('%s(%s);' % (func, tmp_cname))
+        code.putln("if (%s) {" % code.unlikely("%s != -1" % failed_dim_temp))
+        code.putln('%s(%s);' % (func, failed_dim_temp))
         code.putln(code.error_goto(pos))
         code.putln('}')
-        code.funcstate.release_temp(tmp_cname)
+        code.funcstate.release_temp(failed_dim_temp)
     elif negative_indices:
         # Only fix negative indices.
-        for signed, cname, shape in zip(index_signeds, index_cnames,
-                                        entry.get_buf_shapevars()):
+        for signed, cname, shape in zip(index_signeds, index_cnames, entry.get_buf_shapevars()):
             if signed != 0:
                 code.putln("if (%s < 0) %s += %s;" % (cname, cname, shape))
 
@@ -627,7 +625,7 @@ def mangle_dtype_name(dtype):
             prefix = "nn_"
         else:
             prefix = ""
-        type_decl = dtype.declaration_code("")
+        type_decl = dtype.empty_declaration_code()
         type_decl = type_decl.replace(" ", "_")
         return prefix + type_decl.replace("[", "_").replace("]", "_")
 
@@ -665,7 +663,7 @@ def get_type_information_cname(code, dtype, maxdepth=None):
 
         complex_possible = dtype.is_struct_or_union and dtype.can_be_complex()
 
-        declcode = dtype.declaration_code("")
+        declcode = dtype.empty_declaration_code()
         if dtype.is_simple_buffer_dtype():
             structinfo_name = "NULL"
         elif dtype.is_struct:
@@ -678,7 +676,7 @@ def get_type_information_cname(code, dtype, maxdepth=None):
             typecode.putln("static __Pyx_StructField %s[] = {" % structinfo_name, safe=True)
             for f, typeinfo in zip(fields, types):
                 typecode.putln('  {&%s, "%s", offsetof(%s, %s)},' %
-                           (typeinfo, f.name, dtype.declaration_code(""), f.cname), safe=True)
+                           (typeinfo, f.name, dtype.empty_declaration_code(), f.cname), safe=True)
             typecode.putln('  {NULL, NULL, 0}', safe=True)
             typecode.putln("};", safe=True)
         else:
