@@ -260,22 +260,17 @@ class FusedCFuncDefNode(StatListNode):
         Genereate Cython code for instance checks, matching an object to
         specialized types.
         """
-        if_ = 'if'
         for specialized_type in normal_types:
             # all_numeric = all_numeric and specialized_type.is_numeric
-            py_type_name = specialized_type.py_type_name()
-            specialized_type_name = specialized_type.specialization_string
-            pyx_code.context.update(locals())
+            pyx_code.context.update(
+                py_type_name=specialized_type.py_type_name(),
+                specialized_type_name=specialized_type.specialization_string,
+            )
             pyx_code.put_chunk(
                 u"""
-                    {{if_}} isinstance(arg, {{py_type_name}}):
-                        dest_sig[{{dest_sig_idx}}] = '{{specialized_type_name}}'
+                    if isinstance(arg, {{py_type_name}}):
+                        dest_sig[{{dest_sig_idx}}] = '{{specialized_type_name}}'; break
                 """)
-            if_ = 'elif'
-
-        if not normal_types:
-            # we need an 'if' to match the following 'else'
-            pyx_code.putln("if 0: pass")
 
     def _dtype_name(self, dtype):
         if dtype.is_typedef:
@@ -398,46 +393,35 @@ class FusedCFuncDefNode(StatListNode):
         to each specialization, which obtains the buffer each time and tries
         to match the format string.
         """
-        if buffer_types:
-            if pyx_code.indenter(u"else:"):
-                # The first thing to find a match in this loop breaks out of the loop
-                if pyx_code.indenter(u"while 1:"):
-                    pyx_code.put_chunk(
-                        u"""
-                            if ndarray is not None:
-                                if isinstance(arg, ndarray):
-                                    dtype = arg.dtype
-                                elif __pyx_memoryview_check(arg):
-                                    arg_base = arg.base
-                                    if isinstance(arg_base, ndarray):
-                                        dtype = arg_base.dtype
-                                    else:
-                                        dtype = None
-                                else:
-                                    dtype = None
+        # The first thing to find a match in this loop breaks out of the loop
+        pyx_code.put_chunk(
+            u"""
+                if ndarray is not None:
+                    if isinstance(arg, ndarray):
+                        dtype = arg.dtype
+                    elif __pyx_memoryview_check(arg):
+                        arg_base = arg.base
+                        if isinstance(arg_base, ndarray):
+                            dtype = arg_base.dtype
+                        else:
+                            dtype = None
+                    else:
+                        dtype = None
 
-                                itemsize = -1
-                                if dtype is not None:
-                                    itemsize = dtype.itemsize
-                                    kind = ord(dtype.kind)
-                                    dtype_signed = kind == 'i'
-                        """)
-                    pyx_code.indent(2)
-                    pyx_code.named_insertion_point("numpy_dtype_checks")
-                    self._buffer_check_numpy_dtype(pyx_code, buffer_types)
-                    pyx_code.dedent(2)
+                    itemsize = -1
+                    if dtype is not None:
+                        itemsize = dtype.itemsize
+                        kind = ord(dtype.kind)
+                        dtype_signed = kind == 'i'
+            """)
+        pyx_code.indent(2)
+        pyx_code.named_insertion_point("numpy_dtype_checks")
+        self._buffer_check_numpy_dtype(pyx_code, buffer_types)
+        pyx_code.dedent(2)
 
-                    for specialized_type in buffer_types:
-                        self._buffer_parse_format_string_check(
-                                pyx_code, decl_code, specialized_type, env)
-
-                    pyx_code.putln(self.no_match)
-                    pyx_code.putln("break")
-                    pyx_code.dedent()
-
-                pyx_code.dedent()
-        else:
-            pyx_code.putln("else: %s" % self.no_match)
+        for specialized_type in buffer_types:
+            self._buffer_parse_format_string_check(
+                    pyx_code, decl_code, specialized_type, env)
 
     def _buffer_declarations(self, pyx_code, decl_code, all_buffer_types):
         """
@@ -481,7 +465,7 @@ class FusedCFuncDefNode(StatListNode):
                  #                                    self._dtype_name(dtype)))
                 decl_code.putln('ctypedef %s %s "%s"' % (dtype.resolve(),
                                                          self._dtype_name(dtype),
-                                                         dtype.declaration_code("")))
+                                                         dtype.empty_declaration_code()))
 
             if buffer_type.dtype.is_int:
                 if str(dtype) not in seen_int_dtypes:
@@ -499,21 +483,27 @@ class FusedCFuncDefNode(StatListNode):
         Specialize fused types and split into normal types and buffer types.
         """
         specialized_types = PyrexTypes.get_specialized_types(arg.type)
-        # Prefer long over int, etc
-        # specialized_types.sort()
+
+        # Prefer long over int, etc by sorting (see type classes in PyrexTypes.py)
+        specialized_types.sort()
+
         seen_py_type_names = set()
         normal_types, buffer_types = [], []
+        has_object_fallback = False
         for specialized_type in specialized_types:
             py_type_name = specialized_type.py_type_name()
             if py_type_name:
                 if py_type_name in seen_py_type_names:
                     continue
                 seen_py_type_names.add(py_type_name)
-                normal_types.append(specialized_type)
+                if py_type_name == 'object':
+                    has_object_fallback = True
+                else:
+                    normal_types.append(specialized_type)
             elif specialized_type.is_buffer or specialized_type.is_memoryviewslice:
                 buffer_types.append(specialized_type)
 
-        return normal_types, buffer_types
+        return normal_types, buffer_types, has_object_fallback
 
     def _unpack_argument(self, pyx_code):
         pyx_code.put_chunk(
@@ -559,13 +549,17 @@ class FusedCFuncDefNode(StatListNode):
             """)
         decl_code.indent()
 
+        pyx_code.put_chunk(
+            u"""
+                from __future__ import absolute_import  # for later numpy import
+            """)
         pyx_code.put_chunk(
             u"""
                 def __pyx_fused_cpdef(signatures, args, kwargs, defaults):
                     # FIXME: use a typed signature - currently fails badly because
                     #        default arguments inherit the types we specify here!
 
-                    dest_sig = [{{for _ in range(n_fused)}}None,{{endfor}}]
+                    dest_sig = [None] * {{n_fused}}
 
                     if kwargs is None:
                         kwargs = {}
@@ -600,12 +594,24 @@ class FusedCFuncDefNode(StatListNode):
                     default_idx=default_idx,
                 )
 
-                normal_types, buffer_types = self._split_fused_types(arg)
+                normal_types, buffer_types, has_object_fallback = self._split_fused_types(arg)
                 self._unpack_argument(pyx_code)
-                self._fused_instance_checks(normal_types, pyx_code, env)
-                self._buffer_checks(buffer_types, pyx_code, decl_code, env)
-                fused_index += 1
 
+                # 'unrolled' loop, first match breaks out of it
+                if pyx_code.indenter("while 1:"):
+                    if normal_types:
+                        self._fused_instance_checks(normal_types, pyx_code, env)
+                    if buffer_types:
+                        self._buffer_checks(buffer_types, pyx_code, decl_code, env)
+                    if has_object_fallback:
+                        pyx_code.context.update(specialized_type_name='object')
+                        pyx_code.putln(self.match)
+                    else:
+                        pyx_code.putln(self.no_match)
+                    pyx_code.putln("break")
+                    pyx_code.dedent()
+
+                fused_index += 1
                 all_buffer_types.update(buffer_types)
 
             if arg.default:
@@ -642,7 +648,9 @@ class FusedCFuncDefNode(StatListNode):
         fragment_code = pyx_code.getvalue()
         # print decl_code.getvalue()
         # print fragment_code
-        fragment = TreeFragment.TreeFragment(fragment_code, level='module')
+        from .Optimize import ConstantFolding
+        fragment = TreeFragment.TreeFragment(
+            fragment_code, level='module', pipeline=[ConstantFolding()])
         ast = TreeFragment.SetPosTransform(self.node.pos)(fragment.root)
         UtilityCode.declare_declarations_in_scope(
             decl_code.getvalue(), env.global_scope())
@@ -729,7 +737,7 @@ class FusedCFuncDefNode(StatListNode):
         if self.py_func:
             args = [CloneNode(default) for default in defaults if default]
             self.defaults_tuple = TupleNode(self.pos, args=args)
-            self.defaults_tuple = self.defaults_tuple.analyse_types(env, skip_children=True)
+            self.defaults_tuple = self.defaults_tuple.analyse_types(env, skip_children=True).coerce_to_pyobject(env)
             self.defaults_tuple = ProxyNode(self.defaults_tuple)
             self.code_object = ProxyNode(self.specialized_pycfuncs[0].code_object)
 
