@@ -21,8 +21,13 @@ docstrings = True
 # (when all memory will be reclaimed anyways).
 generate_cleanup_code = False
 
+# Generate an annotated HTML version of the input source files.
 annotate = False
 
+# When annotating source files in HTML, include coverage information from
+# this file.
+annotate_coverage_xml = None
+
 # This will abort the compilation on the first error occured rather than trying
 # to keep going and printing further error messages.
 fast_fail = False
@@ -244,6 +249,7 @@ directive_scopes = { # defaults to available everywhere
     'c_string_type': ('module',),
     'c_string_encoding': ('module',),
     'type_version_tag': ('module', 'cclass'),
+    'language_level': ('module',),
 }
 
 def parse_directive_value(name, value, relaxed_bool=False):
@@ -251,7 +257,7 @@ def parse_directive_value(name, value, relaxed_bool=False):
     Parses value as an option value for the given name and returns
     the interpreted value. None is returned if the option does not exist.
 
-    >>> print parse_directive_value('nonexisting', 'asdf asdfd')
+    >>> print(parse_directive_value('nonexisting', 'asdf asdfd'))
     None
     >>> parse_directive_value('boundscheck', 'True')
     True
