@@ -5,12 +5,12 @@ from Cython.Compiler.UtilNodes import *
 import Cython.Compiler.Naming as Naming
 
 class TestTreeFragments(CythonTest):
-    
+
     def test_basic(self):
         F = self.fragment(u"x = 4")
         T = F.copy()
         self.assertCode(u"x = 4", T)
-    
+
     def test_copy_is_taken(self):
         F = self.fragment(u"if True: x = 4")
         T1 = F.root
@@ -46,7 +46,7 @@ class TestTreeFragments(CythonTest):
         v = F.root.stats[1].rhs.operand2.operand1
         a = T.stats[1].rhs.operand2.operand1
         self.assertEquals(v.pos, a.pos)
-        
+
     def test_temps(self):
         TemplateTransform.temp_name_counter = 0
         F = self.fragment(u"""
@@ -54,11 +54,10 @@ class TestTreeFragments(CythonTest):
             x = TMP
         """)
         T = F.substitute(temps=[u"TMP"])
-        s = T.stats
-        self.assert_(s[0].expr.name == "__tmpvar_1")
-#        self.assert_(isinstance(s[0].expr, TempRefNode))
-#        self.assert_(isinstance(s[1].rhs, TempRefNode))
-#        self.assert_(s[0].expr.handle is s[1].rhs.handle)
+        s = T.body.stats
+        self.assert_(isinstance(s[0].expr, TempRefNode))
+        self.assert_(isinstance(s[1].rhs, TempRefNode))
+        self.assert_(s[0].expr.handle is s[1].rhs.handle)
 
 if __name__ == "__main__":
     import unittest
