@@ -2,7 +2,12 @@
 GDB extension that adds Cython support.
 """
 
-from __future__ import with_statement
+from __future__ import print_function
+
+try:
+    input = raw_input
+except NameError:
+    pass
 
 import sys
 import textwrap
@@ -54,6 +59,7 @@ PythonObject = 'PythonObject'
 _data_types = dict(CObject=CObject, PythonObject=PythonObject)
 _filesystemencoding = sys.getfilesystemencoding() or 'UTF-8'
 
+
 # decorators
 
 def dont_suppress_errors(function):
@@ -68,6 +74,7 @@ def dont_suppress_errors(function):
 
     return wrapper
 
+
 def default_selected_gdb_frame(err=True):
     def decorator(function):
         @functools.wraps(function)
@@ -84,6 +91,7 @@ def default_selected_gdb_frame(err=True):
         return wrapper
     return decorator
 
+
 def require_cython_frame(function):
     @functools.wraps(function)
     @require_running_program
@@ -95,6 +103,7 @@ def require_cython_frame(function):
         return function(self, *args, **kwargs)
     return wrapper
 
+
 def dispatch_on_frame(c_command, python_command=None):
     def decorator(function):
         @functools.wraps(function)
@@ -115,6 +124,7 @@ def dispatch_on_frame(c_command, python_command=None):
         return wrapper
     return decorator
 
+
 def require_running_program(function):
     @functools.wraps(function)
     def wrapper(*args, **kwargs):
@@ -152,6 +162,7 @@ class CythonModule(object):
         self.lineno_c2cy = {}
         self.functions = {}
 
+
 class CythonVariable(object):
 
     def __init__(self, name, cname, qualified_name, type, lineno):
@@ -161,6 +172,7 @@ class CythonVariable(object):
         self.type = type
         self.lineno = int(lineno)
 
+
 class CythonFunction(CythonVariable):
     def __init__(self,
                  module,
@@ -297,7 +309,7 @@ class CythonBase(object):
         try:
             source_desc, lineno = self.get_source_desc(frame)
         except NoFunctionNameInFrameError:
-            print '#%-2d Unknown Frame (compile with -g)' % index
+            print('#%-2d Unknown Frame (compile with -g)' % index)
             return
 
         if not is_c and self.is_python_function(frame):
@@ -331,15 +343,15 @@ class CythonBase(object):
             func_address = int(str(gdb_value.address).split()[0], 0)
 
         a = ', '.join('%s=%s' % (name, val) for name, val in func_args)
-        print '#%-2d 0x%016x in %s(%s)' % (index, func_address, func_name, a),
+        sys.stdout.write('#%-2d 0x%016x in %s(%s)' % (index, func_address, func_name, a))
 
         if source_desc.filename is not None:
-            print 'at %s:%s' % (source_desc.filename, lineno),
+            sys.stdout.write(' at %s:%s' % (source_desc.filename, lineno))
 
-        print
+        sys.stdout.write('\n')
 
         try:
-            print '    ' + source_desc.get_source(lineno)
+            sys.stdout.write('    ' + source_desc.get_source(lineno))
         except gdb.GdbError:
             pass
 
@@ -369,7 +381,7 @@ class CythonBase(object):
 
         result = {}
         seen = set()
-        for k, v in pyobject_dict.iteritems():
+        for k, v in pyobject_dict.items():
             result[k.proxyval(seen)] = v
 
         return result
@@ -381,10 +393,9 @@ class CythonBase(object):
             typename = '(%s) ' % (value.type,)
 
         if max_name_length is None:
-            print '%s%s = %s%s' % (prefix, name, typename, value)
+            print('%s%s = %s%s' % (prefix, name, typename, value))
         else:
-            print '%s%-*s = %s%s' % (prefix, max_name_length, name, typename,
-                                     value)
+            print('%s%-*s = %s%s' % (prefix, max_name_length, name, typename, value))
 
     def is_initialized(self, cython_func, local_name):
         cyvar = cython_func.locals[local_name]
@@ -394,7 +405,7 @@ class CythonBase(object):
             # Closed over free variable
             if cur_lineno > cython_func.lineno:
                 if cyvar.type == PythonObject:
-                    return long(gdb.parse_and_eval(cyvar.cname))
+                    return int(gdb.parse_and_eval(cyvar.cname))
                 return True
             return False
 
@@ -473,6 +484,7 @@ class CyGDBError(gdb.GdbError):
         args = args or (self.msg,)
         super(CyGDBError, self).__init__(*args)
 
+
 class NoCythonFunctionInFrameError(CyGDBError):
     """
     raised when the user requests the current cython function, which is
@@ -480,6 +492,7 @@ class NoCythonFunctionInFrameError(CyGDBError):
     """
     msg = "Current function is a function cygdb doesn't know about"
 
+
 class NoFunctionNameInFrameError(NoCythonFunctionInFrameError):
     """
     raised when the name of the C function could not be determined
@@ -503,26 +516,31 @@ class CythonParameter(gdb.Parameter):
         if default is not None:
             self.value = default
 
-    def __nonzero__(self):
+    def __bool__(self):
         return bool(self.value)
 
-    __bool__ = __nonzero__ # python 3
+    __nonzero__ = __bool__  # Python 2
+
+
 
 class CompleteUnqualifiedFunctionNames(CythonParameter):
     """
     Have 'cy break' complete unqualified function or method names.
     """
 
+
 class ColorizeSourceCode(CythonParameter):
     """
     Tell cygdb whether to colorize source code.
     """
 
+
 class TerminalBackground(CythonParameter):
     """
     Tell cygdb about the user's terminal background (light or dark).
     """
 
+
 class CythonParameters(object):
     """
     Simple container class that might get more functionality in the distant
@@ -635,7 +653,7 @@ class CyCy(CythonCommand):
             cy_eval = CyEval('cy_eval'),
         )
 
-        for command_name, command in commands.iteritems():
+        for command_name, command in commands.items():
             command.cy = self
             setattr(self, command_name, command)
 
@@ -671,9 +689,8 @@ class CyImport(CythonCommand):
         for arg in string_to_argv(args):
             try:
                 f = open(arg)
-            except OSError, e:
-                raise gdb.GdbError('Unable to open file %r: %s' %
-                                                (args, e.args[1]))
+            except OSError as e:
+                raise gdb.GdbError('Unable to open file %r: %s' % (args, e.args[1]))
 
             t = etree.parse(f)
 
@@ -714,7 +731,7 @@ class CyImport(CythonCommand):
 
                 for marker in module.find('LineNumberMapping'):
                     cython_lineno = int(marker.attrib['cython_lineno'])
-                    c_linenos = map(int, marker.attrib['c_linenos'].split())
+                    c_linenos = list(map(int, marker.attrib['c_linenos'].split()))
                     cython_module.lineno_cy2c[cython_lineno] = min(c_linenos)
                     for c_lineno in c_linenos:
                         cython_module.lineno_c2cy[c_lineno] = cython_lineno
@@ -781,13 +798,13 @@ class CyBreak(CythonCommand):
 
             if len(funcs) > 1:
                 # multiple functions, let the user pick one
-                print 'There are multiple such functions:'
+                print('There are multiple such functions:')
                 for idx, func in enumerate(funcs):
-                    print '%3d) %s' % (idx, func.qualified_name)
+                    print('%3d) %s' % (idx, func.qualified_name))
 
                 while True:
                     try:
-                        result = raw_input(
+                        result = input(
                             "Select a function, press 'a' for all "
                             "functions or press 'q' or '^D' to quit: ")
                     except EOFError:
@@ -799,11 +816,11 @@ class CyBreak(CythonCommand):
                             break_funcs = funcs
                             break
                         elif (result.isdigit() and
-                            0 <= int(result) < len(funcs)):
+                                0 <= int(result) < len(funcs)):
                             break_funcs = [funcs[int(result)]]
                             break
                         else:
-                            print 'Not understood...'
+                            print('Not understood...')
             else:
                 break_funcs = [funcs[0]]
 
@@ -832,10 +849,10 @@ class CyBreak(CythonCommand):
     def complete(self, text, word):
         # Filter init-module functions (breakpoints can be set using
         # modulename:linenumber).
-        names =  [n for n, L in self.cy.functions_by_name.iteritems()
-                        if any(not f.is_initmodule_function for f in L)]
-        qnames = [n for n, f in self.cy.functions_by_qualified_name.iteritems()
-                        if not f.is_initmodule_function]
+        names =  [n for n, L in self.cy.functions_by_name.items()
+                  if any(not f.is_initmodule_function for f in L)]
+        qnames = [n for n, f in self.cy.functions_by_qualified_name.items()
+                  if not f.is_initmodule_function]
 
         if parameters.complete_unqualified:
             all_names = itertools.chain(qnames, names)
@@ -976,7 +993,7 @@ class CyUp(CythonCommand):
             gdb.execute(self._command, to_string=True)
             while not self.is_relevant_function(gdb.selected_frame()):
                 gdb.execute(self._command, to_string=True)
-        except RuntimeError, e:
+        except RuntimeError as e:
             raise gdb.GdbError(*e.args)
 
         frame = gdb.selected_frame()
@@ -1019,7 +1036,7 @@ class CySelect(CythonCommand):
 
         try:
             gdb.execute('select %d' % (stackdepth - stackno - 1,))
-        except RuntimeError, e:
+        except RuntimeError as e:
             raise gdb.GdbError(*e.args)
 
 
@@ -1069,7 +1086,7 @@ class CyList(CythonCommand):
         sd, lineno = self.get_source_desc()
         source = sd.get_source(lineno - 5, lineno + 5, mark_line=lineno,
                                lex_entire=True)
-        print source
+        print(source)
 
 
 class CyPrint(CythonCommand):
@@ -1103,7 +1120,8 @@ class CyPrint(CythonCommand):
             return []
 
 
-sortkey = lambda (name, value): name.lower()
+sortkey = lambda item: item[0].lower()
+
 
 class CyLocals(CythonCommand):
     """
@@ -1124,7 +1142,7 @@ class CyLocals(CythonCommand):
 
         local_cython_vars = cython_function.locals
         max_name_length = len(max(local_cython_vars, key=len))
-        for name, cyvar in sorted(local_cython_vars.iteritems(), key=sortkey):
+        for name, cyvar in sorted(local_cython_vars.items(), key=sortkey):
             if self.is_initialized(self.get_cython_function(), cyvar.name):
                 value = gdb.parse_and_eval(cyvar.cname)
                 if not value.is_optimized_out:
@@ -1156,14 +1174,14 @@ class CyGlobals(CyLocals):
         max_name_length = max(max_globals_len, max_globals_dict_len)
 
         seen = set()
-        print 'Python globals:'
-        for k, v in sorted(global_python_dict.iteritems(), key=sortkey):
+        print('Python globals:')
+        for k, v in sorted(global_python_dict.items(), key=sortkey):
             v = v.get_truncated_repr(libpython.MAX_OUTPUT_LEN)
             seen.add(k)
-            print '    %-*s = %s' % (max_name_length, k, v)
+            print('    %-*s = %s' % (max_name_length, k, v))
 
-        print 'C globals:'
-        for name, cyvar in sorted(module_globals.iteritems(), key=sortkey):
+        print('C globals:')
+        for name, cyvar in sorted(module_globals.items(), key=sortkey):
             if name not in seen:
                 try:
                     value = gdb.parse_and_eval(cyvar.cname)
@@ -1175,7 +1193,6 @@ class CyGlobals(CyLocals):
                                              max_name_length, '    ')
 
 
-
 class EvaluateOrExecuteCodeMixin(object):
     """
     Evaluate or execute Python code in a Cython or Python frame. The 'evalcode'
@@ -1187,10 +1204,8 @@ class EvaluateOrExecuteCodeMixin(object):
         "Fill a remotely allocated dict with values from the Cython C stack"
         cython_func = self.get_cython_function()
 
-        for name, cyvar in cython_func.locals.iteritems():
-            if (cyvar.type == PythonObject and
-                self.is_initialized(cython_func, name)):
-
+        for name, cyvar in cython_func.locals.items():
+            if cyvar.type == PythonObject and self.is_initialized(cython_func, name):
                 try:
                     val = gdb.parse_and_eval(cyvar.cname)
                 except RuntimeError:
@@ -1227,7 +1242,6 @@ class EvaluateOrExecuteCodeMixin(object):
 
         raise gdb.GdbError("There is no Cython or Python frame on the stack.")
 
-
     def _evalcode_cython(self, executor, code, input_type):
         with libpython.FetchAndRestoreError():
             # get the dict of Cython globals and construct a dict in the
@@ -1383,6 +1397,7 @@ cython_info = CythonInfo()
 cy = CyCy.register()
 cython_info.cy = cy
 
+
 def register_defines():
     libpython.source_gdb_script(textwrap.dedent("""\
         define cy step
