@@ -1,15 +1,19 @@
+
 def primes(kmax):
     result = []
     if kmax > 1000:
         kmax = 1000
+
+    p = [0] * 1000
+    k = 0
+    n = 2
     while k < kmax:
         i = 0
         while i < k and n % p[i] != 0:
-            i = i + 1
+            i += 1
         if i == k:
             p[k] = n
-            k = k + 1
+            k += 1
             result.append(n)
-        n = n + 1
+        n += 1
     return result
-
