@@ -23,21 +23,17 @@ except (ImportError, AttributeError):
     IS_CPYTHON = True
     IS_PYPY = False
 
+from io import open as io_open
 try:
     from StringIO import StringIO
 except ImportError:
-    from io import StringIO
+    from io import StringIO  # doesn't accept 'str' in Py2
 
 try:
     import cPickle as pickle
 except ImportError:
     import pickle
 
-try:
-    from io import open as io_open
-except ImportError:
-    from codecs import open as io_open
-
 try:
     import threading
 except ImportError: # No threads, no problems
@@ -71,32 +67,42 @@ except NameError:
 WITH_CYTHON = True
 CY3_DIR = None
 
-from distutils.dist import Distribution
-from distutils.core import Extension
 from distutils.command.build_ext import build_ext as _build_ext
 from distutils import sysconfig
-distutils_distro = Distribution()
 
 
-if sys.platform == 'win32':
-    # TODO: Figure out why this hackery (see http://thread.gmane.org/gmane.comp.python.cython.devel/8280/).
-    config_files = distutils_distro.find_config_files()
-    try: config_files.remove('setup.cfg')
-    except ValueError: pass
-    distutils_distro.parse_config_files(config_files)
+def get_distutils_distro(_cache=[]):
+    if _cache:
+        return _cache[0]
+    # late import to accomodate for setuptools override
+    from distutils.dist import Distribution
+    distutils_distro = Distribution()
+
+    if sys.platform == 'win32':
+        # TODO: Figure out why this hackery (see http://thread.gmane.org/gmane.comp.python.cython.devel/8280/).
+        config_files = distutils_distro.find_config_files()
+        try: config_files.remove('setup.cfg')
+        except ValueError: pass
+        distutils_distro.parse_config_files(config_files)
+
+        cfgfiles = distutils_distro.find_config_files()
+        try: cfgfiles.remove('setup.cfg')
+        except ValueError: pass
+        distutils_distro.parse_config_files(cfgfiles)
+    _cache.append(distutils_distro)
+    return distutils_distro
 
-    cfgfiles = distutils_distro.find_config_files()
-    try: cfgfiles.remove('setup.cfg')
-    except ValueError: pass
-    distutils_distro.parse_config_files(cfgfiles)
 
 EXT_DEP_MODULES = {
-    'tag:numpy' : 'numpy',
-    'tag:pstats': 'pstats',
-    'tag:posix' : 'posix',
-    'tag:array' : 'array',
-    'tag:ipython': 'IPython',
-    'tag:jedi': 'jedi',
+    'tag:numpy':    'numpy',
+    'tag:asyncio':  'asyncio',
+    'tag:pstats':   'pstats',
+    'tag:posix':    'posix',
+    'tag:array':    'array',
+    'tag:coverage': 'Cython.Coverage',
+    'Coverage':     'Cython.Coverage',
+    'tag:ipython':  'IPython',
+    'tag:jedi':     'jedi',
 }
 
 def patch_inspect_isfunction():
@@ -262,6 +268,10 @@ def get_openmp_compiler_flags(language):
     if not gcc_version:
         return None # not gcc - FIXME: do something about other compilers
 
+    # gcc defines "__int128_t", assume that at least all 64 bit architectures have it
+    global COMPILER_HAS_INT128
+    COMPILER_HAS_INT128 = getattr(sys, 'maxsize', getattr(sys, 'maxint', 0)) > 2**60
+
     compiler_version = gcc_version.group(1)
     if compiler_version and compiler_version.split('.') >= ['4', '2']:
         return '-fopenmp', '-fopenmp'
@@ -271,6 +281,8 @@ try:
 except locale.Error:
     pass
 
+COMPILER = None
+COMPILER_HAS_INT128 = False
 OPENMP_C_COMPILER_FLAGS = get_openmp_compiler_flags('c')
 OPENMP_CPP_COMPILER_FLAGS = get_openmp_compiler_flags('cpp')
 
@@ -292,45 +304,6 @@ def _is_py3_before_32(excluded, version):
 VER_DEP_MODULES = {
     # tests are excluded if 'CurrentPythonVersion OP VersionTuple', i.e.
     # (2,4) : (operator.lt, ...) excludes ... when PyVer < 2.4.x
-    (2,4) : (operator.lt, lambda x: x in ['run.extern_builtins_T258',
-                                          'run.builtin_sorted',
-                                          'run.reversed_iteration',
-                                          ]),
-    (2,5) : (operator.lt, lambda x: x in ['run.any',
-                                          'run.all',
-                                          'run.yield_from_pep380',  # GeneratorExit
-                                          'run.generator_frame_cycle', # yield in try-finally
-                                          'run.generator_expressions_in_class',
-                                          'run.absolute_import',
-                                          'run.relativeimport_T542',
-                                          'run.relativeimport_star_T542',
-                                          'run.initial_file_path',  # relative import
-                                          'run.pynumber_subtype_conversion',  # bug in Py2.4
-                                          'build.cythonize_script',  # python2.4 -m a.b.c
-                                          'build.cythonize_script_excludes',  # python2.4 -m a.b.c
-                                          'build.cythonize_script_package',  # python2.4 -m a.b.c
-                                          ]),
-    (2,6) : (operator.lt, lambda x: x in ['run.print_function',
-                                          'run.language_level', # print function
-                                          'run.cython3',
-                                          'run.property_decorator_T593', # prop.setter etc.
-                                          'run.generators_py', # generators, with statement
-                                          'run.pure_py', # decorators, with statement
-                                          'run.purecdef',
-                                          'run.struct_conversion',
-                                          'run.bytearray_coercion',
-                                          'run.bytearraymethods',
-                                          'run.bytearray_ascii_auto_encoding',
-                                          'run.bytearray_default_auto_encoding',
-                                          # memory views require buffer protocol
-                                          'memoryview.relaxed_strides',
-                                          'memoryview.cythonarray',
-                                          'memoryview.memslice',
-                                          'memoryview.numpy_memoryview',
-                                          'memoryview.memoryviewattrs',
-                                          'memoryview.memoryview',
-                                          'run.withstat_py',
-                                          ]),
     (2,7) : (operator.lt, lambda x: x in ['run.withstat_py27', # multi context with statement
                                           'run.yield_inside_lambda',
                                           'run.test_dictviews',
@@ -352,12 +325,14 @@ VER_DEP_MODULES = {
     (3,1): (_is_py3_before_32, lambda x: x in ['run.pyclass_special_methods',
                                                ]),
     (3,3) : (operator.lt, lambda x: x in ['build.package_compilation',
+                                          'run.yield_from_py33',
                                           ]),
-    (3,4,0,'beta',3) : (operator.le, lambda x: x in ['run.py34_signature',
-                                          ]),
+    (3,4): (operator.lt, lambda x: x in ['run.py34_signature',
+                                         ]),
+    (3,5): (operator.lt, lambda x: x in ['run.py35_pep492_interop',
+                                         ]),
 }
 
-COMPILER = None
 INCLUDE_DIRS = [ d for d in os.getenv('INCLUDE', '').split(os.pathsep) if d ]
 CFLAGS = os.getenv('CFLAGS', '').split()
 CCACHE = os.getenv('CYTHON_RUNTESTS_CCACHE', '').split()
@@ -666,12 +641,11 @@ class CythonCompileTestCase(unittest.TestCase):
 
     def setUp(self):
         from Cython.Compiler import Options
-        self._saved_options = [ (name, getattr(Options, name))
-                                for name in ('warning_errors',
-                                             'clear_to_none',
-                                             'error_on_unknown_names',
-                                             'error_on_uninitialized') ]
-        self._saved_default_directives = Options.directive_defaults.items()
+        self._saved_options = [
+            (name, getattr(Options, name))
+            for name in ('warning_errors', 'clear_to_none', 'error_on_unknown_names', 'error_on_uninitialized')
+        ]
+        self._saved_default_directives = list(Options.directive_defaults.items())
         Options.warning_errors = self.warning_errors
         if sys.version_info >= (3, 4):
             Options.directive_defaults['autotestdict'] = False
@@ -841,7 +815,7 @@ class CythonCompileTestCase(unittest.TestCase):
         cwd = os.getcwd()
         os.chdir(workdir)
         try:
-            build_extension = build_ext(distutils_distro)
+            build_extension = build_ext(get_distutils_distro())
             build_extension.include_dirs = INCLUDE_DIRS[:]
             if incdir:
                 build_extension.include_dirs.append(incdir)
@@ -861,10 +835,12 @@ class CythonCompileTestCase(unittest.TestCase):
 
             related_files = self.related_files(test_directory, module)
             self.copy_files(test_directory, workdir, related_files)
+
+            from distutils.core import Extension
             extension = Extension(
                 module,
-                sources = self.source_files(workdir, module, related_files),
-                extra_compile_args = ext_compile_flags,
+                sources=self.source_files(workdir, module, related_files),
+                extra_compile_args=ext_compile_flags,
                 **extra_extension_args
                 )
 
@@ -1265,7 +1241,15 @@ class CythonPyregrTestCase(CythonRunTestCase):
                     module = import_ext(self.module, ext_so_path)
                     sys.stdout.flush() # helps in case of crashes
                     if hasattr(module, 'test_main'):
-                        module.test_main()
+                        # help 'doctest.DocFileTest' find the module path through frame inspection
+                        fake_caller_module_globals = {
+                            'module': module,
+                            '__name__': module.__name__,
+                        }
+                        call_tests = eval(
+                            'lambda: module.test_main()',
+                            fake_caller_module_globals, fake_caller_module_globals)
+                        call_tests()
                         sys.stdout.flush() # helps in case of crashes
                 except (unittest.SkipTest, support.ResourceDenied):
                     result.addSkip(self, 'ok')
@@ -1322,7 +1306,7 @@ def collect_doctests(path, module_prefix, suite, selectors, exclude_selectors):
     def package_matches(dirname):
         if dirname == 'Debugger' and not include_debugger:
             return False
-        return dirname not in ("Mac", "Distutils", "Plex")
+        return dirname not in ("Mac", "Distutils", "Plex", "Tempita")
     def file_matches(filename):
         filename, ext = os.path.splitext(filename)
         blacklist = ['libcython', 'libpython', 'test_libcython_in_gdb',
@@ -1515,21 +1499,27 @@ class VersionDependencyExcluder:
                 return True
         return False
 
+
 class FileListExcluder:
 
-    def __init__(self, list_file):
+    def __init__(self, list_file, verbose=False):
+        self.verbose = verbose
         self.excludes = {}
-        f = open(list_file)
-        try:
-            for line in f.readlines():
+        self._list_file = os.path.relpath(list_file)
+        with open(list_file) as f:
+            for line in f:
                 line = line.strip()
                 if line and line[0] != '#':
                     self.excludes[line.split()[0]] = True
-        finally:
-            f.close()
 
     def __call__(self, testname, tags=None):
-        return testname in self.excludes or testname.split('.')[-1] in self.excludes
+        exclude = (testname in self.excludes
+                   or testname.split('.')[-1] in self.excludes)
+        if exclude and self.verbose:
+            print("Excluding %s because it's listed in %s"
+                  % (testname, self._list_file))
+        return exclude
+
 
 class TagsSelector:
 
@@ -1578,9 +1568,8 @@ def refactor_for_py3(distdir, cy3_dir):
     # need to convert Cython sources first
     import lib2to3.refactor
     from distutils.util import copydir_run_2to3
-    fixers = [ fix for fix in lib2to3.refactor.get_fixers_from_package("lib2to3.fixes")
-               if fix.split('fix_')[-1] not in ('next',)
-               ]
+    with open('2to3-fixers.txt') as f:
+        fixers = [line.strip() for line in f if line.strip()]
     if not os.path.exists(cy3_dir):
         os.makedirs(cy3_dir)
     import distutils.log as dlog
@@ -1725,7 +1714,7 @@ def main():
     parser.add_option("-x", "--exclude", dest="exclude",
                       action="append", metavar="PATTERN",
                       help="exclude tests matching the PATTERN")
-    parser.add_option("--shard_count", dest="shard_count", metavar="N",
+    parser.add_option("-j", "--shard_count", dest="shard_count", metavar="N",
                       type=int, default=1,
                       help="shard this run into several parallel runs")
     parser.add_option("--shard_num", dest="shard_num", metavar="K",
@@ -1773,15 +1762,15 @@ def main():
     parser.add_option("--watermark", dest="watermark", default=None,
                       help="deterministic generated by string")
     parser.add_option("--use_common_utility_dir", default=False, action="store_true")
+    parser.add_option("--use_formal_grammar", default=False, action="store_true")
 
     options, cmd_args = parser.parse_args()
 
     WORKDIR = os.path.abspath(options.work_dir)
 
-    if sys.version_info[0] >= 3:
-        options.doctests = False
-        if options.with_cython:
-            sys.path.insert(0, options.cython_dir)
+    if options.with_cython and sys.version_info[0] >= 3:
+        sys.path.insert(0, options.cython_dir)
+        if sys.version_info[:2] == (3, 2):
             try:
                 # try if Cython is installed in a Py3 version
                 import Cython.Compiler.Main
@@ -1795,12 +1784,7 @@ def main():
                 # hasn't been refactored yet - do it now
                 global CY3_DIR
                 CY3_DIR = cy3_dir = os.path.join(WORKDIR, 'Cy3')
-                if sys.version_info >= (3,1):
-                    refactor_for_py3(DISTDIR, cy3_dir)
-                elif os.path.isdir(cy3_dir):
-                    sys.path.insert(0, cy3_dir)
-                else:
-                    options.with_cython = False
+                refactor_for_py3(DISTDIR, cy3_dir)
 
     if options.watermark:
         import Cython.Compiler.Version
@@ -1820,6 +1804,9 @@ def main():
                 coverage.erase()
                 coverage.start()
 
+    if options.xml_output_dir:
+        shutil.rmtree(options.xml_output_dir, ignore_errors=True)
+
     if WITH_CYTHON:
         global CompilationOptions, pyrex_default_options, cython_compile
         from Cython.Compiler.Main import \
@@ -1832,6 +1819,7 @@ def main():
         Options.generate_cleanup_code = 3   # complete cleanup code
         from Cython.Compiler import DebugFlags
         DebugFlags.debug_temp_code_comments = 1
+        pyrex_default_options['formal_grammar'] = options.use_formal_grammar
 
     if options.shard_count > 1 and options.shard_num == -1:
         import multiprocessing
@@ -1874,8 +1862,11 @@ def runtests(options, cmd_args, coverage=None):
     ROOTDIR = os.path.abspath(options.root_dir)
     WORKDIR = os.path.abspath(options.work_dir)
 
+    xml_output_dir = options.xml_output_dir
     if options.shard_num > -1:
         WORKDIR = os.path.join(WORKDIR, str(options.shard_num))
+        if xml_output_dir:
+            xml_output_dir = os.path.join(xml_output_dir, 'shard-%03d' % options.shard_num)
 
     # RUN ALL TESTS!
     UNITTEST_MODULE = "Cython"
@@ -1913,7 +1904,7 @@ def runtests(options, cmd_args, coverage=None):
         sys.path.insert(0, os.path.split(libpath)[0])
         CFLAGS.append("-DCYTHON_REFNANNY=1")
 
-    if options.xml_output_dir and options.fork:
+    if xml_output_dir and options.fork:
         # doesn't currently work together
         sys.stderr.write("Disabling forked testing to support XML test output\n")
         options.fork = False
@@ -1932,6 +1923,7 @@ def runtests(options, cmd_args, coverage=None):
                 test_bugs = True
 
     selectors = [ string_selector(r) for r in cmd_args ]
+    verbose_excludes = selectors or options.verbosity >= 2
     if not selectors:
         selectors = [ lambda x, tags=None: True ]
 
@@ -1951,19 +1943,25 @@ def runtests(options, cmd_args, coverage=None):
 
     try:
         import jedi
-        if list(map(int, re.findall('[0-9]+', jedi.__version__))) < [0, 8, 1]:
+        if not ([0, 9] <= list(map(int, re.findall('[0-9]+', jedi.__version__ or '0')))):
             raise ImportError
-    except ImportError:
+    except (ImportError, AttributeError, TypeError):
         exclude_selectors.append(RegExSelector('Jedi'))
 
     if options.exclude:
         exclude_selectors += [ string_selector(r) for r in options.exclude ]
 
+    if not COMPILER_HAS_INT128 or not IS_CPYTHON:
+        exclude_selectors += [RegExSelector('int128')]
+
     if options.shard_num > -1:
         exclude_selectors.append(ShardExcludeSelector(options.shard_num, options.shard_count))
 
     if not test_bugs:
-        exclude_selectors += [ FileListExcluder(os.path.join(ROOTDIR, "bugs.txt")) ]
+        exclude_selectors += [
+            FileListExcluder(os.path.join(ROOTDIR, bugs_file_name), verbose=verbose_excludes)
+            for bugs_file_name in ['bugs.txt'] + (['pypy_bugs.txt'] if IS_PYPY else [])
+        ]
 
     if sys.platform in ['win32', 'cygwin'] and sys.version_info < (2,6):
         exclude_selectors += [ lambda x: x == "run.specialfloat" ]
@@ -2028,9 +2026,12 @@ def runtests(options, cmd_args, coverage=None):
             sys.stderr.write("Including CPython regression tests in %s\n" % sys_pyregr_dir)
             test_suite.addTest(filetests.handle_directory(sys_pyregr_dir, 'pyregr'))
 
-    if options.xml_output_dir:
+    if xml_output_dir:
         from Cython.Tests.xmlrunner import XMLTestRunner
-        test_runner = XMLTestRunner(output=options.xml_output_dir,
+        if not os.path.exists(xml_output_dir):
+            try: os.makedirs(xml_output_dir)
+            except OSError: pass  # concurrency issue?
+        test_runner = XMLTestRunner(output=xml_output_dir,
                                     verbose=options.verbosity > 0)
     else:
         test_runner = unittest.TextTestRunner(verbosity=options.verbosity)
@@ -2086,8 +2087,6 @@ def runtests(options, cmd_args, coverage=None):
 if __name__ == '__main__':
     try:
         main()
-    except SystemExit: # <= Py2.4 ...
-        raise
     except Exception:
         traceback.print_exc()
         try:
