@@ -1,6 +1,6 @@
 #define UN_OP(op) const char* operator op () { return "unary "#op; }
-#define POST_UN_OP(op) const char* operator op (int x) { return "post "#op; }
-#define BIN_OP(op) const char* operator op (int x) { return "binary "#op; }
+#define POST_UN_OP(op) const char* operator op (int x) { x++; return "post "#op; }
+#define BIN_OP(op) const char* operator op (int x) { x++; return "binary "#op; }
 
 #define COMMA ,
 
@@ -45,3 +45,11 @@ public:
     BIN_OP(());
 
 };
+
+class TruthClass {
+public:
+  TruthClass() : value(false) {}
+  TruthClass(bool value) : value(value) {}
+  operator bool() { return value; }
+  bool value;
+};
